//
//  CallKitManager.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2023/1/31.
//

#import "CallKitManager.h"
#import <ZPNs/ZPNs.h>
#import <Flutter/Flutter.h>
#import "NSDictionary+safeInvoke.h"
#import "NSMutableDictionary+safeInvoke.h"
#import "UUIDManager.h"
#import "ZPNsSettings.h"
static CallKitManager *_callKitManager = nil;

static CXProviderConfiguration *_cxProviderConfiguration = nil;

static bool _isProduct = false;

@interface CallKitManager()

@property CXProvider *cxProvider;

@property CXCallController *cxCallController;

@property NSString *callID;

@property bool isProduct;

@property unsigned int seq;

@property NSMutableDictionary<NSString *,CXAction *> *actionMap;

@property NSMutableDictionary<NSString *,id> *completeMap;

@end

@implementation CallKitManager


-(void)setInitConfig:(NSDictionary *)configMap{
    NSString *localizedName = [configMap safeObjectForKey:@"localizedName"];
    NSNumber *supportVideo = [configMap safeObjectForKey:@"supportsVideo"];
    NSNumber *maximumCallsPerCallGroup = [configMap safeObjectForKey:@"maximumCallsPerCallGroup"];
    NSNumber *maximumCallGroups = [configMap safeObjectForKey:@"maximumCallGroups"];
    NSArray<NSNumber *> *supportedHandleTypes = [configMap safeObjectForKey:@"supportedHandleTypes"];
    NSString *iconTemplateImageName = [configMap safeObjectForKey:@"iconTemplateImageName"];
    
    _cxProviderConfiguration = [[CXProviderConfiguration alloc] initWithLocalizedName:localizedName];
    if(supportVideo != nil)_cxProviderConfiguration.supportsVideo = [supportVideo boolValue];
    if(maximumCallsPerCallGroup != nil)_cxProviderConfiguration.maximumCallsPerCallGroup = [maximumCallsPerCallGroup unsignedIntegerValue];
    if(maximumCallGroups != nil)_cxProviderConfiguration.maximumCallGroups = [maximumCallGroups unsignedIntegerValue];
    if(supportedHandleTypes != nil)_cxProviderConfiguration.supportedHandleTypes = [[NSSet alloc] initWithArray:supportedHandleTypes];
    if(supportedHandleTypes != nil)_cxProviderConfiguration.iconTemplateImageData = UIImagePNGRepresentation([UIImage imageNamed:iconTemplateImageName]);
}

+(CallKitManager *)getInstance{
    if(!_callKitManager){
        @synchronized (self) {
            if(!_callKitManager){
                _callKitManager = [[CallKitManager alloc] init];
                _callKitManager.actionMap = [[NSMutableDictionary alloc] init];
                _callKitManager.completeMap = [[NSMutableDictionary alloc] init];
                _callKitManager.seq = 0;
            }
        }
    }
    return _callKitManager;
}

+(void)isProduct:(bool) isProduct{
    _isProduct = isProduct;
}

-(void)reportIncomingCallWithTitle:(NSDictionary *)updateMap UUID:(NSString *)uuidSeq{
    CXCallUpdate *update = [self oCXCallUpdate:updateMap];
    NSUUID *uuid = [[UUIDManager getInstance] getUUID:uuidSeq];
    _callKitManager.cxProvider = [[CXProvider alloc] initWithConfiguration:_cxProviderConfiguration];
    [_callKitManager.cxProvider setDelegate:_callKitManager queue:dispatch_get_main_queue()];
    _callKitManager.cxCallController = [[CXCallController alloc] initWithQueue:dispatch_get_main_queue()];
    [self.cxProvider reportNewIncomingCallWithUUID:uuid update:update completion:^(NSError * _Nullable error) {
    }];
}

- (void)reportCallEndedWithUUID:(NSString *)uuidSeq reason:(CXCallEndedReason)endedReason{
    [self.cxProvider reportCallWithUUID:[[UUIDManager getInstance] getUUID:uuidSeq] endedAtDate:nil reason:endedReason];
}

-(void)reportCallUpdateWithUUID:(NSDictionary *)updateMap UUID:(NSString *)uuidSeq{
    CXCallUpdate *update = [self oCXCallUpdate:updateMap];
    NSUUID *uuid = [[UUIDManager getInstance] getUUID:uuidSeq];
    [self.cxProvider reportCallWithUUID:uuid updated:update];
}

-(void)reportOutgoingCallWithUUID:(NSString *)uuidSeq{
    NSUUID *uuid = [[UUIDManager getInstance] getUUID:uuidSeq];
    [self.cxProvider reportOutgoingCallWithUUID:uuid connectedAtDate:nil];
}
#pragma mark - voIP delegate

- (void)pushRegistry:(nonnull PKPushRegistry *)registry didUpdatePushCredentials:(nonnull PKPushCredentials *)pushCredentials forType:(nonnull PKPushType)type {
    NSString *voipToken = [self getHexStringForData:[pushCredentials token] ];
    [[ZPNs shared] setVoipToken:[pushCredentials token] isProduct:self.isProduct];
    NSLog(@"voip token %@",voipToken);
}

- (void)pushRegistry:(PKPushRegistry *)registry didReceiveIncomingPushWithPayload:(PKPushPayload *)payload forType:(PKPushType)type withCompletionHandler:(void (^)(void))completion{
    NSString *seq = [self generateSeqStr];
    NSDictionary *resultDic = @{@"method":@"didReceiveIncomingPushWithPayload",@"payload":payload.dictionaryPayload,@"seq":seq};
    [self.completeMap safeSetObject:completion forKey:seq];
    ZPNsSettings.event(resultDic);
}

//当接收到呼叫重置时 调用的函数，这个函数必须被实现，其不需做任何逻辑，只用来重置状态
- (void)providerDidReset:(CXProvider *)provider{
    ZPNsSettings.event(@{@"method":@"providerDidReset"});
}

//呼叫开始时回调
- (void)providerDidBegin:(CXProvider *)provider{
    ZPNsSettings.event(@{@"method":@"providerDidBegin"});
}
//音频会话激活状态的回调
- (void)provider:(CXProvider *)provider didActivateAudioSession:(AVAudioSession *)audioSession{
    ZPNsSettings.event(@{@"method":@"didActivateAudioSession"});
}
//音频会话停用的回调
- (void)provider:(CXProvider *)provider didDeactivateAudioSession:(AVAudioSession *)audioSession{
    ZPNsSettings.event(@{@"method":@"didDeactivateAudioSession"});
}
//行为超时的回调
- (void)provider:(CXProvider *)provider timedOutPerformingAction:(CXAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    ZPNsSettings.event(@{@"method":@"timedOutPerformingAction",@"seq":seq});
}
//有事务被提交时调用
//如果返回YES 则表示事务被捕获处理 后面的回调都不会调用 如果返回NO 则表示事务不被捕获，会回调后面的函数
- (BOOL)provider:(CXProvider *)provider executeTransaction:(CXTransaction *)transaction{
    return NO;
}
//点击开始按钮的回调
- (void)provider:(CXProvider *)provider performStartCallAction:(CXStartCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    ZPNsSettings.event(@{@"method":@"performStartCallAction",@"seq":seq});
}
//点击接听按钮的回调
- (void)provider:(CXProvider *)provider performAnswerCallAction:(CXAnswerCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    ZPNsSettings.event(@{@"method":@"performAnswerCallAction",@"seq":seq});
}
//点击结束按钮的回调
- (void)provider:(CXProvider *)provider performEndCallAction:(CXEndCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    ZPNsSettings.event(@{@"method":@"performEndCallAction",@"seq":seq});
}
//点击保持通话按钮的回调
- (void)provider:(CXProvider *)provider performSetHeldCallAction:(CXSetHeldCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    ZPNsSettings.event(@{@"method":@"performSetHeldCallAction",@"seq":seq});
}
//点击静音按钮的回调
- (void)provider:(CXProvider *)provider performSetMutedCallAction:(CXSetMutedCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    ZPNsSettings.event(@{@"method":@"performSetMutedCallAction",@"seq":seq});
}
//点击组按钮的回调
- (void)provider:(CXProvider *)provider performSetGroupCallAction:(CXSetGroupCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    ZPNsSettings.event(@{@"method":@"performSetGroupCallAction",@"seq":seq});
}
//DTMF功能回调
- (void)provider:(CXProvider *)provider performPlayDTMFCallAction:(CXPlayDTMFCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    ZPNsSettings.event(@{@"method":@"performPlayDTMFCallAction",@"seq":seq});
}

-(void)ActionFail:(NSString *)seq{
    CXAction *action = [_actionMap safeObjectForKey:seq];
    [action fail];
}

-(void)ActionFullFill:(NSString *)seq{
    CXAction *action = [_actionMap safeObjectForKey:seq];
    [action fulfill];
}

-(void)didReceiveIncomingPushCompletion:(NSString *)seq{
    id completion = [_completeMap safeObjectForKey:seq];
    ((void (^)(void))completion)();
}

#pragma mark - Utils
- (NSString *)getHexStringForData:(NSData *)data
{
    if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 13) {
        
        if (![data isKindOfClass:[NSData class]]) {
            return @"";
        }
        NSUInteger len = [data length];
        char *chars = (char *)[data bytes];
        NSMutableString *hexString = [[NSMutableString alloc]init];
        for (NSUInteger i=0; i<len; i++) {
            [hexString appendString:[NSString stringWithFormat:@"%0.2hhx" , chars[i]]];
        }
        return hexString;
    } else {
         NSString *myToken = [[data description] stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"<>"]];
        myToken = [myToken stringByReplacingOccurrencesOfString:@" " withString:@""];
        return myToken;
    }
}

-(CXCallUpdate *) oCXCallUpdate:(NSDictionary *)updateMap{
    CXCallUpdate *update = [[CXCallUpdate alloc] init];
    if([updateMap safeObjectForKey:@"supportsDTMF"])update.supportsDTMF = [[updateMap safeObjectForKey:@"supportsDTMF"] boolValue];
    if([updateMap safeObjectForKey:@"supportsHolding"])update.supportsHolding = [[updateMap safeObjectForKey:@"supportsHolding"] boolValue];
    if([updateMap safeObjectForKey:@"supportsGrouping"])update.supportsGrouping = [[updateMap safeObjectForKey:@"supportsGrouping"] boolValue];
    if([updateMap safeObjectForKey:@"supportsUngrouping"])update.supportsUngrouping = [[updateMap safeObjectForKey:@"supportsUngrouping"] boolValue];
    if([updateMap safeObjectForKey:@"hasVideo"])update.hasVideo = [[updateMap safeObjectForKey:@"hasVideo"] boolValue];
    if([updateMap safeObjectForKey:@"remoteHandle"]){
        NSDictionary *remoteHandleMap = [updateMap safeObjectForKey:@"remoteHandle"];
        update.remoteHandle = [[CXHandle alloc] initWithType:[[remoteHandleMap safeObjectForKey:@"type"] intValue] value:[remoteHandleMap safeObjectForKey:@"value"]];
    }
    if([updateMap safeObjectForKey:@"localizedCallerName"])update.localizedCallerName = [updateMap safeObjectForKey:@"localizedCallerName"];
    return update;
}

-(NSString *)generateSeqStr{
    ++_seq;
    NSString *seqStr = [NSString stringWithFormat:@"%u",_seq];
    return seqStr;
}

@end
