import 'internal/call_kit_engine.dart';

import 'internal/call_kit_manager.dart';

import 'call_kit_defines.dart';

abstract class CallKit {

  static setInitConfiguration(CXProviderConfiguration configuration){
    return CallKitManager.setInitConfiguration(configuration);
  }

  static CallKitEngine getInstance() {
    return CallKitManager.getInstance();
  }

  /// Report a new incoming call to the system.
  ///
  /// If completion is invoked with a non-nil `error`, the incoming call has been disallowed by the system and will not be displayed, so the provider should not proceed with the call.
  ///
  /// Completion block will be called on delegate queue, if specified, otherwise on a private serial queue.
  Future<void> reportIncomingCall(CXCallUpdate cxCallUpdate,UUID uuid);

  /// Report that an outgoing call started connecting.
  Future<void> reportOutgoingCall(UUID uuid);

  /// Report an update to call information.
  Future<void> reportCallUpdate(CXCallUpdate cxCallUpdate,UUID uuid);

  /// Report that a call ended. A nil value for `dateEnded` results in the ended date being set to now.
  Future<void>reportCallEnded(CXCallEndedReason endedReason,UUID uuid);
}