import 'package:zego_zpns/src/call_kit_defines.dart';
import 'package:zego_zpns/src/internal/zpns_defines_extension.dart';
import 'package:zego_zpns/src/zpns_defines.dart';

class ZPNsConverter {
  static ZPNsConfig cnvZPNsConfigMapToObject(Map configMap) {
    ZPNsConfig zpnsConfig = ZPNsConfig();

    zpnsConfig.enableHWPush = configMap['hw_push'];
    zpnsConfig.enableMiPush = configMap['xiaomi_push'];
    zpnsConfig.enableVivoPush = configMap['vivo_push'];
    zpnsConfig.enableOppoPush = configMap['oppo_push'];
    zpnsConfig.enableFCMPush = configMap['fcm_push'];

    zpnsConfig.miAppID = configMap['miAppID'];
    zpnsConfig.miAppKey = configMap['miAppKEY'];

    zpnsConfig.oppoAppID = configMap['oppoAppID'];
    zpnsConfig.oppoAppSecret = configMap['oppoAppSecret'];
    zpnsConfig.oppoAppKey = configMap['oppoAppKey'];

    zpnsConfig.vivoAppID = configMap['vivoAppID'];
    zpnsConfig.vivoAppKey = configMap['vivoAppKey'];

    zpnsConfig.hwAppID = configMap['hwAppID'];

    return zpnsConfig;
  }

  static Map cnvZPNsConfigObjectToMap(ZPNsConfig config) {
    Map configMap = {};

    configMap['hw_push'] = config.enableHWPush;
    configMap['xiaomi_push'] = config.enableMiPush;
    configMap['vivo_push'] = config.enableVivoPush;
    configMap['oppo_push'] = config.enableOppoPush;
    configMap['fcm_push'] = config.enableFCMPush;

    configMap['miAppID'] = config.miAppID;
    configMap['miAppKEY'] = config.miAppKey;
    configMap['oppoAppID'] = config.oppoAppID;
    configMap['oppoAppSecret'] = config.oppoAppSecret;
    configMap['oppoAppKey'] = config.oppoAppKey;

    configMap['vivoAppID'] = config.vivoAppID;
    configMap['vivoAppKey'] = config.vivoAppKey;

    configMap['hwAppID'] = config.hwAppID;

    return configMap;
  }

  static ZPNsMessage cnvZPNsMessageMapToObject(Map messageMap) {
    ZPNsMessage zpnsMessage = ZPNsMessage(
        pushSourceType: ZPNsPushSourceTypeExtension
            .mapValue[messageMap['pushSourceType']]!);
    zpnsMessage.title = messageMap['title'] ?? "";
    zpnsMessage.content = messageMap['content'] ?? "";
    zpnsMessage.extras = Map<String, Object>.from(messageMap['extras']) ?? {};
    return zpnsMessage;
  }

  static ZPNsRegisterMessage cnvZPNsRegisterMessageMapToObject(Map messageMap) {
    ZPNsRegisterMessage registerMessage = ZPNsRegisterMessage(
        errorCode: messageMap["errorCode"],
        pushID: messageMap["pushID"],
        pushSourceType: ZPNsPushSourceTypeExtension
            .mapValue[messageMap['pushSourceType']]!);
    registerMessage.errorMessage = messageMap['msg'];
    registerMessage.commandResult = messageMap['commandResult'];
    return registerMessage;
  }

  static Map cnvZPNsLocalMessageObjectToMap(ZPNsLocalMessage message) {
    Map msgMap = {};
    msgMap['title'] = message.title;
    msgMap['content'] = message.content;
    msgMap['payload'] = message.payload;
    msgMap['channelID'] = message.channelID;
    msgMap['iOSSound'] = message.iOSSound;
    msgMap['androidSound'] = message.androidSound;
    return msgMap;
  }

  static Map cnvZPNsNotificationChannelToMap(ZPNsNotificationChannel channel) {
    Map channelMap = {};
    channelMap['channelID'] = channel.channelID;
    channelMap['channelName'] = channel.channelName;
    channelMap['androidSound'] = channel.androidSound;
    return channelMap;
  }

  static Map mCXProviderConfiguration(CXProviderConfiguration configuration) {
    Map map = {};
    if(configuration.localizedName != null)map['localizedName'] = configuration.localizedName;
    if(configuration.supportsVideo_ != null)map['supportsVideo'] = configuration.supportsVideo_;
    if(configuration.maximumCallsPerCallGroup_ != null)map['maximumCallsPerCallGroup'] = configuration.maximumCallsPerCallGroup_;
    if(configuration.maximumCallGroups_ != null)map['maximumCallGroups'] = configuration.maximumCallGroups_;
    if(configuration.supportedHandleTypes_ != null){
      List<int> baseList = [];
      for(CXHandleType type in configuration.supportedHandleTypes_!){
        baseList.add(CXHandleTypeExtension.valueMap[type]!);
      }
      map['supportedHandleTypes'] = baseList;
    }
    if(configuration.iconTemplateImageName_ != null)
    map['iconTemplateImageName'] = configuration.iconTemplateImageName_;
    return map;
  }

  static Map mCXHandle(CXHandle cxHandle){
    Map map = {};
    map['type'] = CXHandleTypeExtension.valueMap[cxHandle.type];
    map['value'] = cxHandle.value;
    return map;
  }

  static Map mCXCallUpdate(CXCallUpdate cxCallUpdate) {
    Map map = {};
    map['supportsDTMF'] = cxCallUpdate.supportsDTMF??false;
    map['supportsHolding'] = cxCallUpdate.supportsHolding??false;
    map['supportsGrouping'] = cxCallUpdate.supportsGrouping??false;
    map['supportsUngrouping'] = cxCallUpdate.supportsUngrouping??false;
    map['hasVideo'] = cxCallUpdate.hasVideo??false;
    if(cxCallUpdate.remoteHandle != null){
      map['remoteHandle'] = mCXHandle(cxCallUpdate.remoteHandle!);
    }
    map['localizedCallerName'] = cxCallUpdate.localizedCallerName??"";
    return map;
  }

}

extension CXHandleTypeExtension on CXHandleType {
  static const valueMap = {
    CXHandleType.CXHandleTypeGeneric: 1,
    CXHandleType.CXHandleTypePhoneNumber : 2,
    CXHandleType.CXHandleTypeEmailAddress : 3
  };

  static const mapValue = {
    1:CXHandleType.CXHandleTypeGeneric,
    2:CXHandleType.CXHandleTypePhoneNumber,
    3:CXHandleType.CXHandleTypeEmailAddress
  };
  int get value => valueMap[this] ?? -1;
}

extension CXCallEndedReasonExtension on CXCallEndedReason {
  static const valueMap = {
    CXCallEndedReason.CXCallEndedReasonFailed : 1,
    CXCallEndedReason.CXCallEndedReasonRemoteEnded : 2,
    CXCallEndedReason.CXCallEndedReasonUnanswered : 3,
    CXCallEndedReason.CXCallEndedReasonAnsweredElsewhere: 4,
    CXCallEndedReason.CXCallEndedReasonDeclinedElsewhere: 5
  };

  static const mapValue = {
    1:CXCallEndedReason.CXCallEndedReasonFailed,
    2:CXCallEndedReason.CXCallEndedReasonRemoteEnded,
    3:CXCallEndedReason.CXCallEndedReasonUnanswered,
    4:CXCallEndedReason.CXCallEndedReasonAnsweredElsewhere,
    5:CXCallEndedReason.CXCallEndedReasonDeclinedElsewhere
  };
  int get value => valueMap[this] ?? -1;
}