import 'package:flutter/services.dart';
import 'package:zego_zpns/src/internal/zpns_converter.dart';
import 'package:zego_zpns/src/internal/zpns_event_handler_impl.dart';
import 'package:zego_zpns/src/internal/zpns_settings.dart';
import 'package:zego_zpns/zego_zpns.dart';
import 'dart:io';

class ZPNsEngine implements ZPNs {
  ZPNsEngine() {
    ZPNsEventHandlerImpl.registerEventHandler();
  }

  @override
  Future<void> registerPush({ZPNsIOSEnvironment? iOSEnvironment,bool? enableIOSVoIP}) async {
    if (Platform.isIOS || Platform.isMacOS) {
      if (iOSEnvironment == null) {
        throw PlatformException(
            code: "1",
            message:
                "ZPNsIOSEnvironment can not be Unkown, You must pass in ZPNsIOSEnvironment when invoke registerPush method on iOS platform");
      }
      switch (iOSEnvironment) {
        case ZPNsIOSEnvironment.Development:
          ZPNsSettings.channel.invokeMethod('enableDebug', {'debug': true});
          break;
        case ZPNsIOSEnvironment.Production:
          ZPNsSettings.channel.invokeMethod('enableDebug', {'debug': false});
          break;
      }
    }
    Map extendedData = {"enableIosVoIP":enableIOSVoIP};
    return await ZPNsSettings.channel.invokeMethod('registerPush',{'extendedData':extendedData});
  }

  @override
  Future<void> applyNotificationPermission() async {
    return await ZPNsSettings.channel.invokeMethod('applyNotificationPermission');
  }

  @override
  Future<void> addLocalNotification(ZPNsLocalMessage message) async {
    return await ZPNsSettings.channel.invokeMethod('addLocalNotification',
        {'message': ZPNsConverter.cnvZPNsLocalMessageObjectToMap(message)});
  }

  @override
  Future<void> createNotificationChannel(
      ZPNsNotificationChannel channel) async {
    return await ZPNsSettings.channel.invokeMethod('createNotificationChannel',
        {'channel': ZPNsConverter.cnvZPNsNotificationChannelToMap(channel)});
  }
}
