///Description: ZPNs push configuration class.
///
///Use cases: Add the required configuration items pushed by each vendor and a Boolean value for whether to enable them.
class ZPNsConfig {
  bool enableHWPush = false;

  bool enableMiPush = false;

  bool enableVivoPush = false;

  bool enableOppoPush = false;

  bool enableFCMPush = false;

  String miAppID = "";

  String miAppKey = "";

  String oppoAppID = "";

  String oppoAppSecret = "";

  String oppoAppKey = "";

  String vivoAppID = "";

  String vivoAppKey = "";

  String hwAppID = "";
}

///Description: ZPNs message class.
///
///Use cases: Use the object of this data class to get the details of the push message.
class ZPNsMessage {
  String title = "";
  String content = "";
  Map<String, Object> extras = {};
  ZPNsPushSourceType pushSourceType;
  ZPNsMessage({required this.pushSourceType});
}

class ZPNsLocalMessage {
  String title = "";
  String content = "";
  String payload = "";
  String iOSSound = "";
  String androidSound = "";
  String channelID = "";
}

class ZPNsNotificationChannel {
  String channelID = "";
  String channelName = "";
  String androidSound = "";
}

class ZPNsRegisterMessage {
  String pushID;

  int errorCode;

  ZPNsPushSourceType pushSourceType;

  String errorMessage = "";

  String commandResult = "";

  ZPNsRegisterMessage(
      {required this.errorCode,
      required this.pushID,
      required this.pushSourceType});
}



enum ZPNsIOSEnvironment { Production, Development }

enum ZPNsIOSBackgroundFetchResult {
  /// New data was successfully downloaded.
  NewData,
  /// There was no new data to download.
  NoData,
  /// An attempt to download data was made but that attempt failed.
  Failed}

enum ZPNsPushSourceType { APNs, ZEGO, FCM, HuaWei, XiaoMi, Oppo, Vivo }

void Function(ZPNsIOSBackgroundFetchResult result)? iOSOnThroughMessageReceivedCompletion;

